;##############################
;# Calculator Security System #
;#        by Chicane          #
;#   http://zapo.calc.org     #
;##############################
#include "asm86.h"
#include "ti86abs.inc"
#include "ti86asm.inc"
#include "Ram86.inc"

_getcode		equ		$4068	;Get scancode, but 1000's of clocks faster than GET_KEY.

_dispahl		equ		$4a33	;Displays ahl in decimal.

_runoff			equ		$4AB1	;Disable run indicator.

_runon			equ		$4AAD	;Enable run indicator.

_off			equ		$4101	;Turn off calc.

_load_ram_ahl	equ		$462F	;Converts ahl to ASIC, loads RAM page.

_cphlde         equ     $403C

;BitCheck Equates
PassState		equ		0		;0 its off, 1 its on
OldPass			equ		1		;0 no old pass needed 1 old pass needed
PassEqual		equ		2		;0 if not equal 1 if equal
MemLock			equ		4		;0 if mem lock disabled
PassSucceed		equ		5		;0 exit pressed during password entry 1 pass entered corectly
ExitFlag		equ		6		;0=CSS 1=TIOS

.org $D748

	nop
	jp		DoMenu
	.dw		$0001
	.dw		CSSStr
	.dw		Icon

DoMenu:
	call	_clrScrn
	call	_runoff
	ld		hl,$0003
	ld		(_penCol),hl
	ld		hl,ProgName
	call	_vputs
	ld		hl,$FC00
	call	Invert

	ld		hl,$0823
	ld		(_penCol),hl
	ld		hl,ChangePassStr
	call	_vputs
	ld		hl,$0E22
	ld		(_penCol),hl
	ld		hl,TurnPassStr
	call	_vputs
	call	OnorOff
	ld		hl,$1423
	ld		(_penCol),hl
	ld		hl,ReturnWhere
	call	_vputs
	call	CSSorExit
	ld		hl,$1A2A
	ld		(_penCol),hl
	ld		hl,PowerDownStr
	call	_vputs
	ld		hl,$201B
	ld		(_penCol),hl
	ld		hl,LockMemStr
	call	_vputs
	call	MemLockCheck
	ld		a,$FF
	ld		hl,$FF00
	ld		(hl),a
	ld		de,$FF01
	ld		bc,143
	ldir
	call	InvertBar

GetKey:
	call	_getcode
	cp		K_EXIT
	jp		z,Exit
	cp		K_ENTER
	jp		z,WhereIsBar
	cp		K_SECOND
	jp		z,WhereIsBar
	cp		K_UP
	jp		z,MoveUp
	cp		K_DOWN
	jp		z,MoveDown
	jr		GetKey

MoveUp:
	call    InvertBar
	ld      hl,Pos
	ld		a,(hl)
	cp		1
	call    z,JumpDown
	dec		a
	ld		(hl),a
	call    InvertBar
	jr		GetKey	
JumpDown:
	ld		a,6
	ret

MoveDown:
	call    InvertBar
	ld      hl,Pos
	ld		a,(hl) 
	cp		5
	call    z,JumpUp
	inc		a
	ld		(hl),a
	call    InvertBar
	jr		GetKey
JumpUp:
	ld		a,0
	ret

WhereIsBar:
	ld		hl,Pos
	ld		a,(hl)
	cp		1
	jp		z,GetNewPass
	cp		2
	jp		z,TogglePass
	cp		3
	jp		z,ToggleExit
	cp		4
	jp		z,ShutDown
	cp		5
	jp		z,ToggleMemLock
	jp		GetKey

TogglePass:
	call	InvertBar
	ld		hl,$0E22
	ld		(_penCol),hl
	ld		hl,TurnPassStr
	call	_vputs
	ld		hl,BitChecks
	bit		PassState,(hl)
	jr		z,TurnPassOn
	res		PassState,(hl)
	jr		FinishToggle
TurnPassOn:
	set		PassState,(hl)
FinishToggle:
	call	OnorOff
	call	InvertBar
	jp		GetKey

OnorOff:
	ld		hl,BitChecks
	bit		PassState,(hl)
	jr		z,DispOn
DispOff:
	ld		hl,OffStr
	call	_vputs
	ret
DispOn:
	ld		hl,OnStr
	call	_vputs
	ret

ToggleExit:
	call	InvertBar
	ld		hl,$1423
	ld		(_penCol),hl
	ld		hl,ReturnWhere
	call	_vputs
	ld		hl,BitChecks
	bit		ExitFlag,(hl)
	jr		z,TurnExitOn
	res		ExitFlag,(hl)
	jr		ReDispExit
TurnExitOn:
	set		ExitFlag,(hl)
ReDispExit:
	call	CSSorExit
	call	InvertBar
	jp		GetKey

CSSorExit:
	ld		hl,BitChecks
	bit		ExitFlag,(hl)
	jr		z,DispCSS
DispZAP:
	ld		hl,ZAPStr
	call	_vputs
	ret
DispCSS:
	ld		hl,CSSStr
	call	_vputs
	ret

ToggleMemLock:
	call	InvertBar
	ld		hl,$201B
	ld		(_penCol),hl
	ld		hl,LockMemStr
	call	_vputs
	ld		hl,BitChecks
	bit		MemLock,(hl)
	jr		z,TurnMemLockOn
	res		MemLock,(hl)
	res		6,(iy+$24)
	jr		FinishToggle
TurnMemLockOn:
	set		MemLock,(hl)
	call	KeyCatcher
FinishMemToggle:
	call	MemLockCheck
	call	InvertBar
	jp		GetKey

MemLockCheck:
	ld		hl,BitChecks
	bit		MemLock,(hl)
	jr		z,DispTurnMemLockOn
	ld		hl,OffStr
	call	_vputs
	ret
DispTurnMemLockOn:
	ld		hl,OnStr
	call	_vputs
	ret

InvertBar:
	ld      a,(Pos)
	ld      h,a
	ld      l,$60
	call    MUL_HL
	ld      de,$FC20
	add     hl,de			
	call    Invert
	ret

Invert:
	ld      b,$70

InvertLoop:
	ld      a,(hl)

	cpl
	ld      (hl),a

	inc     hl

	djnz	InvertLoop

	ret

ShutDown:
	ld		hl,BitChecks
	bit		PassState,(hl)
	jr		nz,ShutDownWP
	res		3,(iy+8)

	set		4,(iy+8)

	call 	_off

	ld		hl,BitChecks
	bit		ExitFlag,(hl)
	jp		z,GetKey
	jp		Exit

ShutDownWP:
	ld		hl,BitChecks
	bit		OldPass,(hl)
	jr		nz,ShutDownCalc
	ld		hl,$3117
	ld		(_penCol),hl
	ld		hl,NoPassSpecified
	set		3,(iy+5)
	call	_vputs
	res		3,(iy+5)
	call	$4d43
	call	ClearPromptArea
	jp		GetKey
	
ShutDownCalc:
	res		3,(iy+8)

	set		4,(iy+8)

	call 	_off

	ld		hl,$3102
	ld		(_penCol),hl
	ld		hl,PassPrompt
	set		3,(iy+5)
	call	_vputs
	res		3,(iy+5)
	call	GetPassword	
	call	ClearPromptArea
	call	ClearEntryArea
	ld		hl,BitChecks
	bit		PassSucceed,(hl)
	jp		z,PressedExit
	call	ComparePWDS
	bit		PassEqual,(hl)
	jp		nz,CorrectPass
PressedExit:
	ld		hl,$311B
	ld		(_penCol),hl
	ld		hl,PasswordIncorrect
	set		3,(iy+5)
	call	_vputs
	res		3,(iy+5)
	call	$4d43
	call	ClearPromptArea
	jp		ShutDownCalc

CorrectPass:
	ld		hl,BitChecks
	bit		ExitFlag,(hl)
	jp		z,GetKey

Exit:
	ld		hl,ProgVAT
	rst		20h
	rst		10h
	jr		c,ExitNoSave
	ld		a,b
	ex		de,hl
	call	_load_ram_ahl
	in		a,(5)
	push	af
	in		a,(6)
	out		(5),a	
	inc		a
	out		(6),a
	ld		de,$4000
	sbc		hl,de
	ld		de,Pos-$D748+4
	add		hl,de
	ld		bc,18
	ex		de,hl
	ld		hl,Pos
	ldir
	pop		af
	out		(5),a
ExitNoSave:
	call	_homeup
	call	_clrScrn
	ret

;#######################################
:# Compare 2 strings                   #
;#######################################
;# Inputs:  de= length                 #
;# 			hl= pointer to PW          #
;# Output:  BitChecks.PassEqual=1 or 0 #
;#######################################

ComparePWDS:
	ld		hl,MainPass
	ld		ix,PassBuffer
	ld		b,15
CompareLoop:
	ld		a,(hl)
	ld		c,(ix)
	cp		c
	jr		nz,PassIncorrect
	inc		hl
	inc		ix
	djnz	CompareLoop
	ld		hl,BitChecks
	set		PassEqual,(hl)
	ret
PassIncorrect:
	ld		hl,BitChecks
	res		PassEqual,(hl)
	ret
	
GetNewPass:
	ld		hl,BitChecks
	bit		OldPass,(hl)
	jr		z,JustNewPass
GetOldPass:	
	ld		hl,$3102
	ld		(_penCol),hl
	ld		hl,OldPassPrompt
	set		3,(iy+5)
	call	_vputs
	res		3,(iy+5)
	call	GetPassword	
	call	ClearPromptArea
	call	ClearEntryArea
	ld		hl,BitChecks
	bit		PassSucceed,(hl)
	jp		z,GetKey
	call	ComparePWDS
	bit		PassEqual,(hl)
	jp		nz,JustNewPass
	ld		hl,$311B
	ld		(_penCol),hl
	ld		hl,PasswordIncorrect
	set		3,(iy+5)
	call	_vputs
	res		3,(iy+5)
	call	$4d43
	call	ClearPromptArea
	jp		GetKey
	
JustNewPass:
	ld		hl,$3102
	ld		(_penCol),hl
	ld		hl,NewPassPrompt
	set		3,(iy+5)
	call	_vputs
	res		3,(iy+5)
	call	GetPassword	
	call	ClearPromptArea
	call	ClearEntryArea
	ld		hl,BitChecks
	bit		PassSucceed,(hl)
	jp		z,GetKey
	set		OldPass,(hl)
	ld		hl,PassBuffer
	ld		de,MainPass
	ld		bc,15
	ldir
	ld		hl,$311B
	ld		(_penCol),hl
	ld		hl,PasswordChanged
	set		3,(iy+5)
	call	_vputs
	res		3,(iy+5)
	call	$4d43
	call	ClearPromptArea
	jp		GetKey

;##########################Clear The Prompt Area######################
ClearPromptArea:
	ld		a,$FF
	ld		hl,$FF00
	ld		(hl),a
	ld		de,$FF01
	ld		bc,143
	ldir
	ret	

;##########################Clear the Password Entry Area###############

ClearEntryArea:
	ld		a,0
	ld		hl,$FFA0
	ld		(hl),a
	ld		de,$FFA1
	ld		bc,111
	ldir
	ret

;##########################Password Entry Routine######################
GetPassword:
	ld		hl,PassBuffer
	ld		(PassPtr),hl
	ld		b,34
	ld		hl,PassBuffer
ClearBuffer:
	ld		(hl),0
	inc		hl
	djnz	ClearBuffer
PutPrompt:
	ld		hl,$3902
	ld		(_penCol),hl
	ld		a,'|'
	call	_vputmap

GetLoop:
	call	_getcode
	cp		K_NOKEY
	jr		z,GetLoop
	cp		K_DEL
	jp		z,BackUp
	cp		K_EXIT
	jp		z,ExitPassLoop
	cp		K_ENTER
	jp		z,EndOfPass
	cp		K_SECOND
	jp		z,EndOfPass
	ld		hl,(PassPtr)
	ld		de,PassBuffer+14
	sbc 	hl,de
	jp 		nc,GetLoop
	ld		hl,(PassPtr)
	ld		(hl),a
	inc		hl
	ld		(PassPtr),hl
	ld		de,PassBuffer
	sbc		hl,de
	ex		de,hl
	ld		hl,DisplayBuffer
	add		hl,de
	ld		a,'*'
	ld		(hl),a
	ld		hl,$3902
	ld		(_penCol),hl
	ld		hl,DisplayBuffer
	call	_vputs
	ld		a,'|'
	call	_vputmap
	jr		GetLoop

BackUp:
	ld		hl,(PassPtr)
	ld		de,PassBuffer
	sbc		hl,de
	jp		z,GetLoop
	ld		hl,(PassPtr)
	dec		hl
	ld		(PassPtr),hl
	ld		(hl),0
	ld		de,PassBuffer
	sbc		hl,de	
	ex		de,hl
	ld		hl,DisplayBuffer
	add		hl,de
	ld		(hl),0
	call	ClearEntryArea
	ld		hl,$3902
	ld		(_penCol),hl
	ld		hl,DisplayBuffer
	call	_vputs
	ld		a,'|'
	call	_vputmap
	jp		GetLoop

EndOfPass:
	ld		hl,PassBuffer
	ld		hl,BitChecks
	set		PassSucceed,(hl)
	ret	

ExitPassLoop:
	ld		hl,BitChecks
	res		PassSucceed,(hl)
	ret

ProgVAT:
	.db $00,$06,"CalcSS"

ProgName:
	.db "Calculator Security System v.2",0

NewPassPrompt:
	.db "New Password:",0

OldPassPrompt:
	.db "Old Password:",0

PassPrompt:
	.db "Password:",0

ChangePassStr:
	.db "Change Password",0

TurnPassStr:
	.db "Turn Password  ",0

PowerDownStr:
	.db "Power Down",0

PasswordChanged:
	.db "Password Changed!",0

PasswordIncorrect:
	.db "Password Incorrect!",0

NoPassSpecified:
	.db "No Password Specified!",0

PassNotEnabled:
	.db "Password Not Enabled!",0

Icon:
	.db 8,1
	.db %01000011
	.db %10000001
	.db %10011000
	.db %01010011
	.db %00001010
	.db %00011001
	.db %10000011
	.db %11000000

OnStr:
	.db "On   ",0

OffStr:
	.db "Off",0

ReturnWhere:
	.db "Return to  ",0

ZAPStr:
	.db "ZAP-OS   ",0
CSSStr:
	.db "Calc-SS",0

LockMemStr:
	.db "Turn Memory Lock  ",0

Pos:
	.db	1
BitChecks:
	.db 0
MainPass:
	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

PassPtr:
	.dw	0000
PassBuffer:
	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

DisplayBuffer:
	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	
KeyCatcher:
	ld		hl,$5a92
	rst		20h
	rst		10h
	call	nc,_delvar
	ld		hl,code_end- code_start
 	call 	_CREATEPROG
 	ld		a,b
 	ex		de,hl
 	call	$4c3f
 	call	_SET_ABS_DEST_ADDR
 	xor		a
 	ld		hl,code_end - code_start
 	call	_SET_MM_NUM_BYTES
 	ld		hl,code
 	call	_SET_ABS_SRC_ADDR
 	call	_mm_ldir
 	set		6,(iy+$24)
 	ret

code:
.org _asm_exec_ram-2
code_start:
.db $8e,$28
 call $479f
 ld a,(_asm_reg_a)
 push af
 ld a,(_CXCURAPP)

check_delete:
 cp $23
 jr nz,check_resets
 pop af
clear_key_then_leave:
 xor a
 ld a,K_EXIT

skip_key:
 cp a					;set zero flag
 ret					;quit [sqrt]key

check_resets:
	cp	$1d
	jr	z,thrownexit
	cp	$1e
	jr	z,thrownexit
	cp	$1f
	jr	z,thrownexit
	cp	$1b
	jr	z,throwlink
	pop af
	cp	a
	ret

thrownexit:
	pop	af
	ld	a,K_F5
	cp	a
	ret

throwlink:
	pop	af
	ld	a,K_EXIT
	cp	a
	ret

code_end:

.end
