;+---------------------------+
;|                           |
;|         TiL v1.11         |
;|                           |
;+---------------------------+
;A 583-bytes Asm text viewer 4 your Ti-86,
;by Stefano Bonora <ste@softhome.net>
;C-Parser By Roberto Boati
;
;TiL Features:
;- Scrolling
;- Ram paging (allowing strings of arbitrary lenght)
;- VAT Support
;- Strings chooser (aehm...)
;- Bookmarks
;- Contrast changing
;
;To Do's:
;- Substring search
;- Words Hilighting.
;- A little smaller TiL
;
;Future features (Hoping ^_^):
;- Compression (Bloat?) support
;- Advanced (menu driven) bookmarks support
;- Better Interface (TiL v1.0 is _very_ rough)
;- Shell incapsulation. <= This could save me a lot of 'Better Interface'
;  work... any volunteer? ^_^
;
;Coming Soon, from Roberto Boati:
;- TiLe, the multi-platform Tcl/Tk TiL Editor, with mathematical symbols, text
;  formatting and stuff like that.. Stay tuned!! (We need just a font editor
;  to draw the 86 fonts, and it's done!!)
;
;N.B.: TiL stands 4 Texas Instruments Less. (Less is a popular Unix text
;      viewer)
;
;Til is NOT optimized for high speed, I tried to code it with space-saving in
;my mind.
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include "asm86.h"
#include "ti86asm.inc"
#include "ti86math.inc"			; I know, a lot of useless stuff.. but
#include "ti86ops.inc"			; who cares? :)
#include "ti86abs.inc"
#include "ti86und.inc"
EOF	= $03
AT	= $40	;the '@' symbol
SWAP_RAM7	= $47F3
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;Strings Choice
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
.org _asm_exec_ram
Begin:
	call BUSY_OFF
	call SWAP_RAM7
	ld hl, $BFFF
SearchStr:
	ld de, (VAT_END)
	inc de
	or a
	sbc hl, de
	jp c, Exit2			;If (VAT_END) > hl there are not other strings, exit
	add hl, de			;else restore hl
	ld a,(hl)
	cp $0C
	jr z, StrFound		;Ehy, it's a string!
itsnotastring:			;if this entry doesn't point to a string
 	dec hl				;Let's skip AHL
	dec hl
	dec hl
	dec hl
	dec hl
 	ld b,(hl)			;and the variable's name
 	inc b
SkipName:
	dec hl
	djnz SkipName
	jr SearchStr		;one more time! maybe we'll be more lucky this time
StrFound:
	dec hl				;string found: let's take its AHL
	ld e, (hl)			;storing 'L' in e
	dec hl
	ld d, (hl)			;storing 'H' in d
	push de				;saving de in stack
	dec hl
	ld a, (hl)			;Saving  'A' in mem
	ld (BOF), a
	dec hl
	dec hl
	ld b, (hl)			;b=namelenght
	ld ix, STRNAME		;ix=place for name-inverting 
InvName:
	dec hl
	ld a, (hl)
	ld (ix), a
	inc ix
	djnz InvName
	ld (ix), $0			;End of inverted Name
	dec hl
	push hl				;saving current position
	call _clrLCD		;(_clrLCD destroys everything)
	xor a
	ld (_penCol), a		;
	ld (_penRow), a
	ld b, 4				;# of prompt-lines 
	ld c, 1
	ld hl, PROMPT		;
	call IncLine		;Writes keys' explanation
Prompt:
	ld a, 30			; put the name in the middle of screen
	ld (_penRow), a		;6=fonts' height
	ld a, 10
	ld (_penCol), a
	ld hl, STRNAME
	call _vputs			;displays variable's name
Choice:
	halt
	call GET_KEY
	cp K_ENTER
	jr z, Prep			;Prepares the string
	cp K_EXIT
	jr z, Exit
	cp K_NOKEY
	jr z, Choice
	pop hl				;get back current VAT position
	pop de				;de is useless, since we didn't select this string
	jp SearchStr		;Any other key will continue the search
Prep:
	pop hl				;get back useless VAT position
	pop de
	ld a, (BOF)			;get back A: A+
Preamb:
	ex de, hl			;			+H+L= AHL
	call _load_ram_ahl	;swaps in the right page and sets hl
	in a, (6)			;Useless.. maybe
	ld (CURRAMPG), a
	ld (BOF), a
	inc hl
	inc hl
	ld (BOF+1), hl		;now we are ready to begin...
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;The Real Thing
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
DrawPage:
	push hl
	call _clrLCD
	pop hl
	ld b, 10			;10=# of lines fitting the screen
	ld c, 1				;c=1 ->IncLine writes chars on screen
	call IncLine		;The routine that writes/skips line
Key:
	halt
	push hl
nobook:					;When there are no more bookmarks we will land here
	call GET_KEY
	pop hl				;Restore hl, destroyed by GET_KEY
	cp K_DOWN
	jr z, LineDn
	cp K_UP
	jr z, LineUp
	cp K_F1
	jr z, PageUp
	cp K_F2				;I've put here PgDn's code.
	jr nz, F3
	ld a, EOF
	cp (hl)
	jr nz, DrawPage
	jr Key				;End of PgDn code
F3:
	cp K_F3				;Err... This key clears the screen
	jp z, Boss			;until is pressed again ;)
	cp K_F4
	jr z, BoF
	cp K_F5
	jr z, EoF
	cp K_MINUS			;Contrast Changing
	jr z, ContrD
	cp K_PLUS
	jr z, ContrU
	cp K_EXIT
	jp z, Begin
	cp K_RIGHT
	jr z, BookM			;Jumps to next bookmark
	jr Key
Exit:
	pop hl
	pop bc
Exit2:
	call _clrLCD
	ret
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;Text Movement
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
LineDn:
	ld b, 10	;DecLine and IncLine's parameters will be explained later
	call DecLine
	jr DrawPage
LineUp:
	ld b,12		;(# of lines to back-skip)+1
callDL:			; EoF and PageUp will land here... just to save some byte
	call DecLine
	jr DrawPage
PageUp:
	ld b,21		;(# of lines to back-skip)+1
	jr callDL
EoF:
	ld c, $0	;Skip-line mode
Keepon:
	ld b, $FF		;
	call IncLine	;IncLine takes care of ram-swapping too
	ld a, $03
	cp (hl)
	jr nz, Keepon
	ld b, 11
	jr callDL
BoF:
	ld a, (BOF)		;sets the page of the 1st line of thestring
	out (6), a
	ld (CURRAMPG), a	;updates CURRAMPG (=BOF)
	ld hl, (BOF+1)
	jp DrawPage
Boss:
	xor a
	out (2), a				;sets contrast to zero
cicle:						;waits 4 F3
	halt
	push hl
	call GET_KEY
	pop hl
	cp K_F3
	jr nz, cicle
	ld a, ($C008)			;here is stroed the contrast value
	out (2), a				;restore previous contrast
	jp Key
ContrD:
	ld a, ($C008)
	dec a
	jr com
ContrU:
	ld a, ($C008)
	inc a
com:					;From here ContrD&ContrU are the same
	jp c, Key			;if c, we have gone too far.
	out (2), a			;else, set new contrast
	ld ($C008), a			;update ($C008)
	jp Key				;and returns to key loop
BookM:
	push hl					; if there are not other bookmarks we 
seek:						; will restore the previous position
	ld b, $1	; # of line to skip
	ld c, $0	; do not write anything
	call IncLine
	ld a, EOF	; tests if we have reached the end of the string
	cp (hl)		; (= no more bookmarks)
	jp z, nobook
	ld a, AT	; AT=$40='@'
	cp (hl)		; tests if the beginning of the string is a '@'
	jr nz, seek
	pop bc		; the old hl value we pushed b4 is useless now, we use bc as our trashcan
	jp DrawPage
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;Main Routines
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;DecLine's parameters: b= # of line to back skip
;Decline returns: hl = beginning of the whished line.
;
;IncLine's parameters: b= # of line to skip
;                      c= if 0 don't write anything, else write on screen 
;                         at penrow, pencol
;IncLine returns: hl = beginning of the wished line.
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
DecLine:
	push hl
	pop ix				;ix=hl
Dfiletest:
	ld a, (BOF+1)		;are we at the beginning of the string? (BOF+1)== l?
	cp l
	jr nz, Dpagetest	;no, let's go testing if we are at ram-page's end
	ld a, (BOF+2)		;(BOF+2)=h? (continuing 'top of string' test)
	cp h
	jr nz, Dpagetest	;no: there is no problem, go to Ram-page checking.
	ld a, (BOF)			;in (BOF) is stored the beginning ram page of the string
	push hl				;if also this value matches, we are at Top of the string
	ld hl, CURRAMPG
	cp (hl)
	pop hl				;restoring hl value to point the passed value
	ret z				;hl & Ram page match: we are at the beginning of the string!
Dpagetest:	;are we at the beginning of Ram-page?
	xor a
	cp l
	jr nz, Dchartest
	ld a, $80
	cp h
	jr nz, Dchartest
	call SwapBack
	push hl
	pop ix
	xor a
	cp (hl)
	jr nz, Dchartest
	djnz Dchartest
	call SwapForw
	ret
Dchartest:	;are we at the beginning of the string?
	xor a
	cp (ix-1)
	dec ix
	push ix
	pop hl
	jr nz, Dfiletest
	djnz Dfiletest
	inc hl
	ret
;-=-=-
IncLine:
	xor a
	ld (_penCol), a
	ld (_penRow), a
Ifiletest:
	ld a, $03
	cp (hl)
	ret z
Ipagetest:
	xor a
	cp l
	jr nz, Ilinetest
	ld a, $C0
	cp h
	jr nz, Ilinetest
	call SwapForw
Ilinetest:
	xor a
	cp (hl)
	jr z, eol
	cp c
	jr z, inchl
	ld a, (hl)
	call _vputmap
inchl:
	inc hl
	jr Ifiletest
eol:
	inc hl
	xor a
	ld (_penCol), a
	ld a, (_penRow)
	add a, 6
	ld (_penRow), a		;6=fonts' height
	djnz Ifiletest
	ret
;-=-=
SwapBack:
	in a, (6)
	dec a
	out (6), a
	ld (CURRAMPG), a
	ld hl, $BFFF
	ret
SwapForw:
	in a, (6)
	inc a
	out (6), a
	ld (CURRAMPG), a
	ld hl, $8000
	ret
;-=-=-=-=-=
CURRAMPG:
.db $0
BOF:
.db $0
.db $0
.db $0
STRNAME:
.db $0	
.db $0	
.db $0	
.db $0	
.db $0	
.db $0	
.db $0	
.db $0
.db $0
PROMPT:
.db "Press:",0
.db "-ENTER to display the string",0
.db "-EXIT to...exit :)",0
.db "-ANY other to skip",0
.end
